/* ++++++++++
	scsiprobe.c
+++++ */

#include <OS.h>
#include <Drivers.h>
#include <KernelExport.h>
#include <PCI.h>
#include <stdlib.h>
#include "scsiprobe_driver.h"


/* ----------
	init_driver

	Called when driver is loaded.  One-time initializations go here.
----- */
long
init_driver (void)
{
	return B_NO_ERROR;
}


/* ----------
	scsi_open - called on each open().
----- */

static long
scsi_open (device_info* /* dev */, ulong /* flags */)
{
	return B_NO_ERROR;
}


/* ----------
	scsi_close
----- */

static long
scsi_close (device_info* /* dev */)
{
	return B_NO_ERROR;
}


/* ----------
	scsi_read - does nothing
----- */

static long
scsi_read (device_info* /* dev */, void* /* buf */,
		   ulong /* count */, ulong /* pos */)
{
	return 0;
}


/* ----------
	scsi_write - does nothing
----- */

static long
scsi_write (device_info* /* dev */, void* /* buf */,
			ulong /* count */, ulong /* pos */)
{
	return 0;
}


/* ----------
	scsi_control
----- */

static long
scsi_control (device_info* /* dev */, ulong msg, void *buf)
{
	long					result = B_ERROR;
	CCB_HEADER 				*ccbh;
	CCB_SCSIIO				*ccb;
	scsiprobe_inquiry		*data;
	scsiprobe_reset			*reset;
	scsiprobe_path_inquiry	*path;

	switch (msg) {
		case SCSIPROBE_INQUIRY:
			if (!(ccbh = xpt_ccb_alloc()))
				return result;
			data = (scsiprobe_inquiry *)buf;
			ccbh->cam_path_id = data->path;
			ccbh->cam_target_id = data->id;
			ccbh->cam_target_lun = data->lun;

			ccb = (CCB_SCSIIO *)ccbh;

			ccbh->cam_flags = CAM_DIR_IN + CAM_DIS_DISCONNECT;
    		ccb->cam_dxfer_len = data->len;
			ccb->cam_data_ptr = (uchar *)&data->data;
			ccb->cam_cdb_len = 6;						/* scsi cmd len */
    		ccb->cam_cdb_io.cam_cdb_bytes[0] = 0x12;	/* INQUIRY cmd */
    		ccb->cam_cdb_io.cam_cdb_bytes[1] =			/* lun */
					ccbh->cam_target_lun << 5;
			ccb->cam_cdb_io.cam_cdb_bytes[2] = 0x00;	/* page code */
			ccb->cam_cdb_io.cam_cdb_bytes[3] = 0x00;	/* reserved */
			ccb->cam_cdb_io.cam_cdb_bytes[4] = data->len;
			ccb->cam_cdb_io.cam_cdb_bytes[5] = 0x00;	/* control */

			xpt_action (ccbh);

			if (ccbh->cam_status == CAM_REQ_CMP)
				result = B_NO_ERROR;
			xpt_ccb_free (ccbh);
			return result;

		case SCSIPROBE_RESET:
			if (!(ccbh = xpt_ccb_alloc()))
				return result;
			reset = (scsiprobe_reset *)buf;
			ccbh->cam_path_id = reset->path;
			ccbh->cam_func_code = XPT_RESET_BUS;
			xpt_action (ccbh);

			if (ccbh->cam_status == CAM_REQ_CMP)
				result = B_NO_ERROR;
			xpt_ccb_free (ccbh);
			return result;

		case SCSIPROBE_VERSION:
			return 2;

		case SCSIPROBE_HIGHEST_PATH:
			if (!(ccbh = xpt_ccb_alloc()))
				return result;
			ccbh->cam_path_id = 0xff;
			ccbh->cam_func_code = XPT_PATH_INQ;
			xpt_action (ccbh);

			if (ccbh->cam_status == CAM_REQ_CMP)
				result = ((CCB_PATHINQ *)ccbh)->cam_hpath_id;
			xpt_ccb_free (ccbh);
			return result;

		case SCSIPROBE_PATH_INQUIRY:
			if (!(ccbh = xpt_ccb_alloc()))
				return result;
			path = (scsiprobe_path_inquiry *)buf;
			ccbh->cam_path_id = path->path;
			ccbh->cam_func_code = XPT_PATH_INQ;
			xpt_action (ccbh);

			if (ccbh->cam_status == CAM_REQ_CMP) {
				memcpy(&path->data, ccbh, sizeof(CCB_PATHINQ));
				result = B_NO_ERROR;
			}
			xpt_ccb_free (ccbh);
			return result;

		default:
			return B_ERROR;
	}
	return B_NO_ERROR;
}


/* -----
	driver-related structures
----- */

device_entry	devices[] = {
	{
		"/dev/scsiprobe",	/* name */
		scsi_open, 			/* -> open entry point */
		scsi_close, 		/* -> close entry point */
		scsi_control, 		/* -> control entry point */
		scsi_read,			/* -> read entry point */
		scsi_write			/* -> write entry point */
	},
	0
};
